@@ -2,11 +2,11 @@ from trilogy.core.constants import ALL_ROWS_CONCEPT, INTERNAL_NAMESPACE
 from trilogy.core.models.build import BuildGrain
 from trilogy.core.models.environment import Environment
 from trilogy.core.processing.concept_strategies_v3 import (
-    GroupNode,
     History,
     search_concepts,
 )
 from trilogy.core.processing.node_generators import gen_group_node
+from trilogy.core.processing.nodes import GroupNode
 
 
 def test_group_node(test_environment, test_environment_graph):
@@ -53,7 +53,7 @@ def test_group_node_property(test_environment: Environment, test_environment_gra
     }
     assert input_concept_names == {
         "category_name_length",
-        "category_name",
+        # "category_name",
         "category_id",
     }
     final = group_node.resolve()
@@ -85,7 +85,7 @@ def test_group_node_property_all(test_environment: Environment, test_environment
     }
     assert input_concept_names == {
         "category_name_length",
-        "category_name",
+        # "category_name",
         "category_id",
     }
     final = group_node.resolve()
