@@ -64,13 +64,13 @@ def datasource_to_node(input: BuildDatasource) -> str:
 
 
 class ReferenceGraph(nx.DiGraph):
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, **kwargs) -> None:
         super().__init__(*args, **kwargs)
         self.concepts: dict[str, BuildConcept] = {}
         self.datasources: dict[str, BuildDatasource] = {}
         self.pseudonyms: set[tuple[str, str]] = set()
 
-    def copy(self):
+    def copy(self) -> "ReferenceGraph":
         g = ReferenceGraph()
         g.concepts = self.concepts.copy()
         g.datasources = self.datasources.copy()
@@ -83,7 +83,7 @@ class ReferenceGraph(nx.DiGraph):
         # g.add_edges_from(self.edges(data=True))
         return g
 
-    def remove_node(self, n):
+    def remove_node(self, n) -> None:
         if n in self.concepts:
             del self.concepts[n]
         if n in self.datasources:
@@ -93,40 +93,14 @@ class ReferenceGraph(nx.DiGraph):
     def add_node(self, node_for_adding, fast: bool = False, **attr):
         if fast:
             return super().add_node(node_for_adding, **attr)
-        if isinstance(node_for_adding, BuildConcept):
-            node_name = concept_to_node(node_for_adding)
-            self.concepts[node_name] = node_for_adding
-        elif isinstance(node_for_adding, BuildDatasource):
-            node_name = datasource_to_node(node_for_adding)
-            self.datasources[node_name] = node_for_adding
-        else:
-            node_name = node_for_adding
-            if attr.get("datasource"):
-                self.datasources[node_name] = attr["datasource"]
+        node_name = node_for_adding
+        if attr.get("datasource"):
+            self.datasources[node_name] = attr["datasource"]
         super().add_node(node_name, **attr)
 
-    def add_edge(self, u_of_edge, v_of_edge, fast: bool = False, **attr):
-        if fast:
-            return super().add_edge(u_of_edge, v_of_edge, **attr)
-        if isinstance(u_of_edge, BuildConcept):
-            orig = u_of_edge
-            u_of_edge = concept_to_node(u_of_edge)
-            if u_of_edge not in self.nodes:
-                self.add_node(orig)
-        elif isinstance(u_of_edge, BuildDatasource):
-            origd = u_of_edge
-            u_of_edge = datasource_to_node(u_of_edge)
-            if u_of_edge not in self.nodes:
-                self.add_node(origd)
+    def add_datasource_node(self, node_name, datasource) -> None:
+        self.datasources[node_name] = datasource
+        super().add_node(node_name, datasource=datasource)
 
-        if isinstance(v_of_edge, BuildConcept):
-            orig = v_of_edge
-            v_of_edge = concept_to_node(v_of_edge)
-            if v_of_edge not in self.nodes:
-                self.add_node(orig)
-        elif isinstance(v_of_edge, BuildDatasource):
-            origd = v_of_edge
-            v_of_edge = datasource_to_node(v_of_edge)
-            if v_of_edge not in self.nodes:
-                self.add_node(origd)
-        super().add_edge(u_of_edge, v_of_edge)
+    def add_edge(self, u_of_edge, v_of_edge, fast: bool = False, **attr):
+        return super().add_edge(u_of_edge, v_of_edge, **attr)
