@@ -204,6 +204,15 @@ class Executor(object):
         output.append(compiled_sql)
         return output
 
+    @generate_sql.register
+    def _(self, command: ProcessedShowStatement) -> List[str]:
+        output = []
+        for statement in command.output_values:
+            if isinstance(statement, (ProcessedQuery, ProcessedQueryPersist)):
+                compiled_sql = self.generator.compile_statement(statement)
+                output.append(compiled_sql)
+        return output
+
     @generate_sql.register  # type: ignore
     def _(self, command: MultiSelectStatement) -> List[str]:
         output = []
@@ -410,6 +419,13 @@ class Executor(object):
                 )
                 output.extend(results)
                 continue
+            elif isinstance(statement, ProcessedValidateStatement):
+                validate_result = handle_processed_validate_statement(
+                    statement, self.generator, self.validate_environment
+                )
+                if validate_result:
+                    output.append(validate_result)
+                continue
             if non_interactive:
                 if not isinstance(
                     statement, (ProcessedCopyStatement, ProcessedQueryPersist)
