@@ -379,14 +379,16 @@ class ParseToObjects(Transformer):
     def start(self, args):
         return args
 
+    def LINE_SEPARATOR(self, args):
+        return MagicConstants.LINE_SEPARATOR
+
     def block(self, args):
         output = args[0]
         if isinstance(output, ConceptDeclarationStatement):
-            if len(args) > 1 and isinstance(args[1], Comment):
-                output.concept.metadata.description = (
-                    output.concept.metadata.description
-                    or args[1].text.split("#")[1].strip()
-                )
+            if len(args) > 1 and args[1] != MagicConstants.LINE_SEPARATOR:
+                comments = [x for x in args[1:] if isinstance(x, Comment)]
+                merged = "\n".join([x.text.split("#")[1].rstrip() for x in comments])
+                output.concept.metadata.description = merged
         # this is a bad plan for now;
         # because a comment after an import statement is very common
         # and it's not intuitive that it modifies the import description
@@ -913,7 +915,7 @@ class ParseToObjects(Transformer):
         return Comment(text=args[0].value)
 
     def PARSE_COMMENT(self, args):
-        return Comment(text=args.value)
+        return Comment(text=args.value.rstrip())
 
     @v_args(meta=True)
     def select_transform(self, meta: Meta, args) -> ConceptTransform:
@@ -990,6 +992,9 @@ class ParseToObjects(Transformer):
     def order_by(self, args):
         return OrderBy(items=args[0])
 
+    def over_component(self, args):
+        return ConceptRef(address=args[0].value.lstrip(",").strip())
+
     def over_list(self, args):
         return [x for x in args]
 
@@ -1000,7 +1005,7 @@ class ParseToObjects(Transformer):
     def validate_statement(self, meta: Meta, args) -> ValidateStatement:
         if len(args) == 2:
             scope = args[0]
-            targets = args[1]
+            targets = args[1].split(",")
         elif len(args) == 0:
             scope = ValidationScope.ALL
             targets = None
@@ -2293,7 +2298,9 @@ def parse_text(
             raise _create_syntax_error(210, pos, text)
 
         # Handle FROM token error
-        parsed_tokens = [x.value for x in e.token_history] if e.token_history else []
+        parsed_tokens = (
+            [x.value for x in e.token_history if x] if e.token_history else []
+        )
         if parsed_tokens == ["FROM"]:
             raise _create_syntax_error(101, pos, text)
 
